from __future__ import absolute_import

import json

from jsonschema import validate, ValidationError as JSONSchemaValidationError

from django import forms
from django.core.exceptions import ValidationError


class SchemaValidator(object):
    def __init__(self, schema):
        self.schema = schema
    
    def __call__(self, value):
        try:
            decoded_value = json.loads(value)
        except ValueError as error:
            raise ValidationError(str(error))
        try:
            validate(decoded_value, self.schema)
        except JSONSchemaValidationError as error:
            raise ValidationError('%s: %s' %('.'.join(error.path), error.message))
        return value

class JSONSchemaField(forms.CharField):
    '''
    A django form field that takes in a schema definition and serializes the result in JSON.
    Renders to a textarea with a data-schemajson attribute containing the initial json .
    Javascript is loaded to convert the field into an Alpacajs powered form: http://www.alpacajs.org/
    
    Upon return validate the submission using python-jsonschema
    '''
    widget = forms.Textarea
    
    def __init__(self, schema, **kwargs):
        self.schema = schema
        super(JSONSchemaField, self).__init__(**kwargs)
        self.validators.append(SchemaValidator(schema=schema))
    
    def widget_attrs(self, widget):
        attrs = super(JSONSchemaField, self).widget_attrs(widget)
        attrs.update(self.get_data_attributes())
        return attrs
    
    def get_data_attributes(self):
        return {
            'data-schemajson': json.dumps(self.schema)
        }
    
    #TODO make the js handling pluggable
    class Media:
        js = ('http://www.alpacajs.org/js/alpaca.min.js',)
        css = {
            'all': ('http://www.alpacajs.org/css/alpaca.min.css',)
        }

#CONSIDER: a JSONSchemaForm that has a schema attribute and constructs the appropriate base_fields with JSONSchemaFields for the complex subfields.
#class MyForm(JSONSchemaForm): schema=schema; afield=forms.CharField()
#model forms infer that we should save these values and should be custom, likely utilizing a single JSONSchemaField



#TODO find a better submodule name
from django.forms import widgets, fields


def pretty_name(name):
    """Converts 'first_name' to 'First name'"""
    if not name:
        return u''
    return name.replace('_', ' ').capitalize()

class DjangoFormToJSONSchema(object):
    def convert_form(self, form, json_schema=None):
        if json_schema is None:
            json_schema = {
                #'title':dockit_schema._meta
                #'description'
                'type':'object',
                'properties':{}, #TODO SortedDict
            }
        #CONSIDER: base_fields when given a class, fields for when given an instance
        for name, field in form.base_fields.iteritems():
            json_schema['properties'][name] = self.convert_formfield(name, field, json_schema)
        return json_schema
    
    input_type_map = {
        'text': 'string',
    }
    
    def convert_formfield(self, name, field, json_schema):
        #TODO detect bound field
        widget = field.widget
        target_def = {
            'title': field.label or pretty_name(name),
            'description': field.help_text,
        }
        if field.required:
            target_def['required'] = [name] #TODO this likely is not correct
        #TODO JSONSchemaField; include subschema and ref the type
        if isinstance(field, fields.URLField):
            target_def['type'] = 'string'
            target_def['format'] = 'url'
        elif isinstance(field, fields.FileField):
            target_def['type'] = 'string'
            target_def['format'] = 'uri'
        elif isinstance(field, fields.DateField):
            target_def['type'] = 'string'
            target_def['format'] = 'date'
        elif isinstance(field, fields.DateTimeField):
            target_def['type'] = 'string'
            target_def['format'] = 'datetime'
        elif isinstance(field, (fields.DecimalField, fields.FloatField)):
            target_def['type'] = 'number'
        elif isinstance(field, fields.IntegerField):
            target_def['type'] = 'integer'
        elif isinstance(field, fields.EmailField):
            target_def['type'] = 'string'
            target_def['format'] = 'email'
        elif isinstance(field, fields.NullBooleanField):
            target_def['type'] = 'boolean'
        elif isinstance(widget, widgets.CheckboxInput):
            target_def['type'] = 'boolean'
        elif isinstance(widget, widgets.Select):
            if widget.allow_multiple_selected:
                target_def['type'] = 'array'
            else:
                target_def['type'] = 'string'
            target_def['enum'] = [choice[0] for choice in field.choices]
        elif isinstance(widget, widgets.Input):
            translated_type = self.input_type_map.get(widget.input_type, 'string')
            target_def['type'] = translated_type
        else:
            target_def['type'] = 'string'
        return target_def
        
class DjangoModelToJSONSchema(DjangoFormToJSONSchema):
    def convert_model(self, model, json_schema=None):
        model_form = None #TODO convert to model form
        #TODO handle many2many and inlines
        return self.convert_form(model_form, json_schema)

#TODO move to django-dockit
class DocKitSchemaToJSONSchema(DjangoFormToJSONSchema):
    def convert_dockitschema(self, dockit_schema, json_schema=None):
        if json_schema is None:
            json_schema = {
                #'title':dockit_schema._meta
                #'description'
                'type':'object',
                'properties':{}, #TODO SortedDict
            }
        for key, field in dockit_schema._meta.fields.iteritems():
            json_schema['properties'][key] = self.convert_dockitfield(key, field, json_schema)
        return json_schema
    
    def convert_dockitfield(self, name, field, json_schema):
        #if simple field, get the form field
        if True: #TODO is simple
            formfield = field.formfield()
            return self.convert_formfield(formfield, json_schema)
        #else: #complex stuff
        target_def = {}




import json

from django.utils import unittest
from django import forms

from djangojsonschema.jsonschema import DjangoFormToJSONSchema
from djangojsonschema.forms import JSONSchemaField


class SurveyForm(forms.Form):
    name = forms.CharField()
    ranking = forms.ChoiceField(choices=[('1', 'excellent'), ('2', 'average'), ('3', 'poor')])

class JSONSchemaFieldTestCase(unittest.TestCase):
    def setUp(self):
        self.encoder = DjangoFormToJSONSchema()
        self.schema = self.encoder.convert_form(SurveyForm)
        
        class TargetForm(forms.Form):
            ticket = forms.CharField()
            
            #presumable a very configurable survey can be plugged in here
            survey_entry = JSONSchemaField(schema=self.schema)
        
        self.targetform = TargetForm
    
    def test_field_presentation(self):
        field = self.targetform()['survey_entry']
        html = field.as_widget()
        self.assertTrue('data-schemajson' in html)
    
    def test_field_validation(self):
        survey_response = {
            'name': 'John Smith',
            'ranking': '1',
        }
        post = {
            'survey_entry': json.dumps(survey_response),
            'ticket':'text'
        }
        form = self.targetform(data=post)
        self.assertTrue(form.is_valid(), str(form.errors))
        
        survey_response = {
            'name': 'John Smith',
            'ranking': '5',
        }
        post = {
            'survey_entry': json.dumps(survey_response),
            'ticket':'text'
        }
        form = self.targetform(data=post)
        self.assertFalse(form.is_valid())
        print form.errors



#!/usr/bin/env python
import os
try:
    from setuptools import setup, find_packages
except ImportError:
    from distutils.core import setup, find_packages

VERSION = '0.2.0'
PATH = os.path.dirname(os.path.abspath(__file__))
try:
    LONG_DESC = '\n===='+open(os.path.join(PATH, 'README.rst'), 'r').read().split('====', 1)[-1]
except IOError: #happens when using tox
    LONG_DESC = ''

setup(name='django-jsonschema',
      version=VERSION,
      description="django-jsonschema converts Django Forms into JSON Schema compatibile representations",
      long_description=LONG_DESC,
      classifiers=[
          'Programming Language :: Python',
          'Environment :: Web Environment',
          'Framework :: Django',
          'Operating System :: OS Independent',
          'Natural Language :: English',
          'Development Status :: 4 - Beta',
          'Intended Audience :: Developers',
          'License :: OSI Approved :: BSD License',
          'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
      ],
      keywords='django json schema',
      author = 'Jason Kraus',
      author_email = 'zbyte64@gmail.com',
      maintainer = 'Jason Kraus',
      maintainer_email = 'zbyte64@gmail.com',
      url='http://github.com/celerityweb/django-jsonschema',
      license='New BSD License',
      packages=find_packages(exclude=['tests']),
      test_suite='tests.runtests.runtests',
      tests_require=(
        'pep8',
        'coverage',
        'django',
        'Mock',
        'nose',
        'django-nose',
      ),
      include_package_data = True,
      zip_safe = False,
  )



from jsonschema import Draft4Validator

from django.utils import unittest
from django import forms

from djangojsonschema.jsonschema import DjangoFormToJSONSchema


check_schema = Draft4Validator.check_schema

CHOICES = [('first', 'first'), ('second', 'second')]

class TestForm(forms.Form):
    a_charfield = forms.CharField(help_text='Any string')
    a_textarea = forms.CharField(widget=forms.Textarea, help_text='Any paragraph')
    url = forms.URLField()
    a_boolean = forms.BooleanField()
    select_option = forms.ChoiceField(choices=CHOICES)
    a_date = forms.DateField()
    a_datetime = forms.DateTimeField()
    a_decimal = forms.DecimalField()
    an_email = forms.EmailField()
    a_file = forms.FileField()
    #a_filepath = forms.FilePathField()
    a_float = forms.FloatField()
    an_image = forms.ImageField()
    an_integer = forms.IntegerField()
    an_ipaddress = forms.IPAddressField()
    #a_generic_ipaddress = forms.GenericIPAddressField()
    a_multiple_choice = forms.MultipleChoiceField(choices=CHOICES)
    a_typed_multiple_choice = forms.TypedMultipleChoiceField(choices=CHOICES)
    a_null_boolean = forms.NullBooleanField() #not sure what this should be
    a_regex = forms.RegexField(regex=r'<([A-Z][A-Z0-9]*)\b[^>]*>(.*?)</\1>') #matches tags
    a_slug = forms.SlugField()
    a_time = forms.TimeField()

class FormToJsonSchemaTestCase(unittest.TestCase):
    def setUp(self):
        self.encoder = DjangoFormToJSONSchema()
    
    def test_convert_form(self):
        form_repr = self.encoder.convert_form(TestForm)
        print form_repr
        try:
            check_schema(form_repr)
        except Exception as error:
            print error.message
            print error.path
            print error.validator
            print error.cause
            raise
        instance_form_repr = self.encoder.convert_form(TestForm())
        check_schema(instance_form_repr)
    
    def test_convert_charfield(self):
        name = 'a_charfield'
        ideal_repr = {
            'required': [name],
            'type': 'string',
            'description': u'Any string',
            'title': 'A charfield',
        } 
        field = TestForm.base_fields[name]
        json_schema = {'properties':{}}
        schema_repr = self.encoder.convert_formfield(name, field, json_schema)
        self.assertEqual(schema_repr, ideal_repr)
    
    def test_convert_charfield(self):
        #CONSIDER: the json spec doesn't define a textarea, this is an option of alpacajs
        name = 'a_textarea'
        ideal_repr = {
            'required': [name],
            'type': 'string',
            'description': u'Any paragraph',
            'title': 'A textarea',
        } 
        field = TestForm.base_fields[name]
        json_schema = {'properties':{}}
        schema_repr = self.encoder.convert_formfield(name, field, json_schema)
        self.assertEqual(schema_repr, ideal_repr)
    
    def test_convert_urlfield(self):
        name = 'url'
        ideal_repr = {
            'required': [name],
            'type': 'string',
            'format': 'url',
            'description': u'',
            'title': 'Url',
        } 
        field = TestForm.base_fields[name]
        json_schema = {'properties':{}}
        schema_repr = self.encoder.convert_formfield(name, field, json_schema)
        self.assertEqual(schema_repr, ideal_repr)
    
    def test_convert_booleanfield(self):
        name = 'a_boolean'
        ideal_repr = {
            'required': [name],
            'type': 'boolean',
            'description': u'',
            'title': 'A boolean',
        } 
        field = TestForm.base_fields[name]
        json_schema = {'properties':{}}
        schema_repr = self.encoder.convert_formfield(name, field, json_schema)
        self.assertEqual(schema_repr, ideal_repr)
    
    def test_convert_select_option(self):
        name = 'select_option'
        ideal_repr = {
            'required': [name],
            'type': 'string',
            'description': u'',
            'title': 'Select option',
            'enum': ['first', 'second'],
        } 
        field = TestForm.base_fields[name]
        json_schema = {'properties':{}}
        schema_repr = self.encoder.convert_formfield(name, field, json_schema)
        self.assertEqual(schema_repr, ideal_repr)
    
    def test_convert_date(self):
        name = 'a_date'
        ideal_repr = {
            'required': [name],
            'type': 'string',
            'format': 'date',
            'description': u'',
            'title': 'A date',
        } 
        field = TestForm.base_fields[name]
        json_schema = {'properties':{}}
        schema_repr = self.encoder.convert_formfield(name, field, json_schema)
        self.assertEqual(schema_repr, ideal_repr)
    
    def test_convert_datetime(self):
        name = 'a_datetime'
        ideal_repr = {
            'required': [name],
            'type': 'string',
            'format': 'datetime',
            'description': u'',
            'title': 'A datetime',
        } 
        field = TestForm.base_fields[name]
        json_schema = {'properties':{}}
        schema_repr = self.encoder.convert_formfield(name, field, json_schema)
        self.assertEqual(schema_repr, ideal_repr)
    
    def test_convert_decimal(self):
        name = 'a_decimal'
        ideal_repr = {
            'required': [name],
            'type': 'number',
            'description': u'',
            'title': 'A decimal',
        } 
        field = TestForm.base_fields[name]
        json_schema = {'properties':{}}
        schema_repr = self.encoder.convert_formfield(name, field, json_schema)
        self.assertEqual(schema_repr, ideal_repr)
    
    def test_convert_email(self):
        name = 'an_email'
        ideal_repr = {
            'required': [name],
            'type': 'string',
            'format': 'email',
            'description': u'',
            'title': 'An email',
        } 
        field = TestForm.base_fields[name]
        json_schema = {'properties':{}}
        schema_repr = self.encoder.convert_formfield(name, field, json_schema)
        self.assertEqual(schema_repr, ideal_repr)
    
    def test_convert_file(self):
        name = 'a_file'
        ideal_repr = {
            'required': [name],
            'type': 'string',
            'format': 'uri',
            'description': u'',
            'title': 'A file',
        } 
        field = TestForm.base_fields[name]
        json_schema = {'properties':{}}
        schema_repr = self.encoder.convert_formfield(name, field, json_schema)
        self.assertEqual(schema_repr, ideal_repr)
    
    def test_convert_float(self):
        name = 'a_float'
        ideal_repr = {
            'required': [name],
            'type': 'number',
            'description': u'',
            'title': 'A float',
        } 
        field = TestForm.base_fields[name]
        json_schema = {'properties':{}}
        schema_repr = self.encoder.convert_formfield(name, field, json_schema)
        self.assertEqual(schema_repr, ideal_repr)
    
    def test_convert_integer(self):
        name = 'an_integer'
        ideal_repr = {
            'required': [name],
            'type': 'integer',
            'description': u'',
            'title': 'An integer',
        } 
        field = TestForm.base_fields[name]
        json_schema = {'properties':{}}
        schema_repr = self.encoder.convert_formfield(name, field, json_schema)
        self.assertEqual(schema_repr, ideal_repr)

